# libasa

Библиотека для управления аналоговым сигнатурным анализатором. Позволяет подключиться к серверно-аппаратной части, управлять настройками оборудования и получать сигнатуры.

## Инструкция для Linux

### 1. Установка libcurl, cmake

Для сборки/установки под Linux понадобится CMake и CURL. Если они не установлены, смотрите инструкцию по установке в папке `deps/linux`.

### 2. Сборка и установка

Обратите внимание, что пользователь, который собирает библиотеку, должен быть владельцем каталога проекта (и всех подкаталогов/файлов). В файловой системе, в которой лежит проект, должна быть возможность менять права доступа к файлам (это нужно, чтобы запустить build-скрипты для зависимых библиотек).

Для сборки создайте отдельную папку в папке проекта:
```bash
mkdir build
cd build
```
Сгенерируйте make-файлы в эту папку:
```bash
cmake ..
```

Соберите весь проект и установите собранную библиотеку:
```bash
make
sudo make install
```

Готово. В папке `build` появится библиотека `libasa.so` и исполняемый файл `asaexample`. Библиотека `libasa.so` также появится в системных путях `/usr/local/lib` и будет доступна для линковки из других проектов.

Иногда нужно собрать библиотеку без установки в систему, для этого выполните:
```bash
make
```
Запуск:
```bash
export LD_LIBRARY_PATH=/usr/local/lib
./asaexample
```


## Инструкция для Windows

Для сборки под Windows понадобятся CMake, MinGW (или MSVC) и Redistributable Packages 2013. Для генерации файлов проектом удобно использовать [CMake GUI](https://cmake.org/download/):

* В CMake GUI в поле "Where is the source code" указать полный путь до папки проекта `md_asa_lib`.

* В CMake GUI в поле  "Where to build the binaries" указать полный путь до каталога, куда положить сборку. Например, это может быть папка `build` в каталоге проекта.

* В CMake GUI нажать кнопку "Configure". В новом окне программа предложит выбрать генератор проекта. Можно выбрать либо MinGW, либо Visual Studio.

Далее, в зависимости от желаемого типа проекта:

*Для Visual Studio 2013*

* Выбрать в поле "Specify the generator for this project" Visual Studio 2013.

* В поле "Optional platform fot generator (if empty, generator uses: Win32)" выбрать Win32, если будет собираться библиотека для Windows 32-bit. Если библиотека будет собираться для Windows 64-bit, выбрать x64.

* Нажать "Finish".

* Нажать "Generate", затем "Open project".

* Откроется Visual Studio с проектом. Дальше собрать с помощью меню "BUILD -> Build Solution", как обычный MSVC Release проект.

* Готово. Можно запускать собранный пример `asaexample.exe`. Библиотеки `asa.dll`, `asa.lib`, `libxmlrpc.dll`, `libxmlrpc_client.dll`, `libxmlrpc_util.dll`, `libxmlrpc_xmlparse.dll`, `libxmlrpc_xmltok.dll` лежат рядом.

*Для MinGW*

* Выбрать MinGW, нажать "Finish".

* Нажать кнопку "Generate".

* Открыть консоль, встать в директорию со сгенерированными файлами.

* Выполнить сборку mingw (путь до mingw должен быть в системных путях). Если библиотека будет собираться для Windows 32-bit:
  
    ```bash
    mingw32-make
    ```
    
    Если библиотека будет собираться для Windows 64-bit:
    
    ```bash
    mingw64-make
    ```
    
* Готово. Можно запускать пример `asaexample.exe`. Библиотеки `asa.dll`, `asa.lib`, `libxmlrpc.dll`, `libxmlrpc_client.dll`, `libxmlrpc_util.dll`, `libxmlrpc_xmlparse.dll`, `libxmlrpc_xmltok.dll` лежат рядом (при сборке с MinGW - `libasa.dll`; статической библиотеки .lib нет).

## Дополнительные инструкции для Windows
Проект использует библиотеку wininet и другие стандартные для Windows библиотеки.

В Windows должны быть установлены распространяемые пакеты MSVC.

Библиотеки XMLRPC лежат в директории проекта уже собранные. Если по каким-то причинам они не подошли вашей системе (битность, "сильно" другая версия Windows и пр.), их можно пересобрать. Для этого зайти в каталог `vendor\xmlrpc-c-1.51.06\Windows\Project\vs2010express\` и открыть решение `vs2010express.sln`. Visual Studio старше 2010 версии предложит "портирвать" эти файлы в новый формат - нужно согласиться с этим предложением. Затем собрать. В зависимости от того, используется ли сборка md_asa_lib MinGW или MSVC, понадобятся статические или динамические библиотеки xmlrpc - лучше собрать обе. Для этого в configuration manager выбрать сборку Static Release, собрать, затем выбрать сборку DLL Release и собрать её.

